#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>
#include <fcntl.h>
  
/*
  spi_conv: program to read SPIDER image files 

  Tobin Fricke 2002-06-12 <tobin@splorg.org>
  Weizmann Institute of Science, Rehovot, Israel

  The image file format used by SPIDER (System for Processing Image Data in
  Electron microscopy and Related fields) is documented:

     http://www.wadsworth.org/spider_doc/spider/docs/image_doc.html

  Strangely the file format uses floating point numbers for everything, even
  for integral quantities such as the pixel dimensions of the image.  As a
  further annoyance, the numerical format is machine-dependent with regards
  to endianess (little versus big) and floating point format (VAX versus IEEE).

  $Id: spi_conv.c,v 1.2 2002/06/24 14:57:14 tobin Exp tobin $

 */

#include "spi_file.h"
#include "pgm_file.h"


int main(int argc, char **argv) {

  int fd;

  float *buf = 0;
  spider_header_t header;

  assert( sizeof(float) == 4 );    
  
  if (argc != 2) { 
    printf("usage: %s filename\n",argv[0]);
    printf("\n This program reads a SPIDER format image and dumps it out as PGM.\n\n");
    exit(0);
  }

  fd = open(argv[1], O_RDONLY | O_NOCTTY );

  if (fd == -1) {
    perror("Couldn't open the file");
    exit(0);
  }

  /* read in the image */

  spider_read(fd, &header, &buf, malloc);

  /* dump the image in PGM format */

  output_pgm( buf, (int)header.nrow, (int)header.nsam );

  /* we're done! */

  free(buf);
  close(fd);
  return 0;
}




