#include "spi_file.h"
#include <unistd.h>
#include <stdio.h>

int spider_read(int fd, spider_header_t *header, float **buf, void *(*malloc)(size_t)) {

  int i;

  /* Read in the SPIDER image header */
  
  read(fd, header, 1024);

  /* It's possible (likely!) that the SPIDER file was created on a machine
     with endianness different from that of the local machine.  For example
     here the SPIDER files are created on SGI IRIX which is big-endian while
     I am trying to read the files on this little-endian x86 linux machine.

     For now I'm just assuming that the file is in big-endian format and am
     converting it to local endianness.  TODO: Detect which endianness is 
     actually in use, in case the file is little-endian. 
  
     Also we only have to convert up to word #211, since after that are
     character strings, which do not suffer from byte order issues.

     --tobin
  */

  /* Convert from network byte order (big endian) to host byte order */

  for (i=0; i<211; i++)
    ((int *)header)[i] = ntohl(((int *)header)[i]);
  
  /* seek to, and read in the image data */

  if (-1 == lseek(fd, (off_t)header->labbyt, SEEK_SET)) {
    perror("couldn't seek to image data");
    return -1;
  }

  *buf = (float *)malloc((size_t)(header->nslice * header->nrow * header->nsam * sizeof(float))); 

  if (buf == NULL) {
    perror("Couldn't allocate memory to hold the image");
    return -1;
  }

  if (-1 == read(fd, *buf,(int)header->nslice * (int)header->nrow * (int)header->nsam * sizeof(float) )) {
    perror("Coudn't read image data");
    return -1;
  }

  /* convert to local byte order */

  for (i=0; i< (int)header->nslice * (int)header->nrow * (int)header->nsam; i++)
    ((int *)(*buf))[i] = ntohl(((int *)(*buf))[i]);

  return 0;
}
