#include "mex.h"

#include "spi_file.h"
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

/*  

This is a matlab MEX function to read in a SPIDER image into MATLAB.

2002-06-24 Tobin Fricke <tobin@splorg.org> Weizmann Institute of Science
$Id: spi_read_matlab.c,v 1.3 2002/06/24 16:27:21 tobin Exp tobin $

*/

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

  int fd;
  float *buf = 0;
  spider_header_t header;
  char filename[256];
  int dims[3] = { 0, 0, 0 };

  if ( sizeof(float) != 4 ) {
    mexErrMsgTxt("C type \"float\" is not 4 bytes.  Recompile with new options.");
  }
    
  if ((nlhs != 1) || (nrhs != 1)) 
    mexErrMsgTxt("Must have exactly one input argument and one output argument.");
  
  if (!mxIsChar(prhs[0])) 
    mexErrMsgTxt("The input argument must be a string (the SPI file's filename)");

  /* there is probably an easy way to do this without copying. */

  mxGetString(prhs[0], filename, sizeof(filename));

  fd = open(filename, O_RDONLY | O_NOCTTY );

  if (fd == -1) {
    mexWarnMsgTxt(strerror(errno));
    mexErrMsgTxt("Couldn't open the file");
  }

  /* read in the image */

  spider_read(fd, &header, &buf, mxMalloc);

  /* somehow return the data to matlab */

  plhs[0] = mxCreateNumericArray(3, dims, mxSINGLE_CLASS, mxREAL);

  dims[2] = header.nslice;  
  dims[0] = header.nrow;
  dims[1] = header.nsam;

  mxSetDimensions(plhs[0], dims, 3);
  mxSetData(plhs[0], buf);

  close(fd);
  return;
}
